;;; finder-inf.el --- keyword-to-package mapping
;; Keywords: help
;;; Commentary:
;; Don't edit this file.  It's generated by finder.el

;;; Code:

(setq finder-package-info '(
    ("abbrev.el"
        "abbrev mode commands for Emacs"
        (abbrev  dumped)
        "lisp/")
    ("about.el"
        "the About The Authors page (shameless self promotion)."
        (extensions)
        "lisp/")
    ("alist.el"
        "utility functions about association-list"
        (alist)
        "lisp/")
    ("apropos.el"
        "apropos commands for users and programmers."
        (help)
        "lisp/")
    ("auto-save.el"
        "Safer autosaving for EFS and tmp."
        (extensions  dumped)
        "lisp/")
    ("auto-show.el"
        "perform automatic horizontal scrolling as point moves"
        (extensions  internal  dumped)
        "lisp/")
    ("autoload.el"
        "maintain autoloads in loaddefs.el."
        (maint)
        "lisp/")
    ("backquote.el"
        "Full backquote support for elisp.  Reverse compatible too."
        (extensions  dumped)
        "lisp/")
    ("blessmail.el"
        "Decide whether movemail needs special privileges."
        (internal)
        "lisp/")
    ("buff-menu.el"
        "buffer menu main function and support functions."
        (extensions  dumped)
        "lisp/")
    ("buffer.el"
        "buffer routines taken from C"
        (internal  dumped)
        "lisp/")
    ("build-report.el"
        "Automatically formatted build reports for XEmacs"
        (internal)
        "lisp/")
    ("byte-optimize.el"
        "the optimization passes of the emacs-lisp byte compiler."
        (internal)
        "lisp/")
    ("bytecomp-runtime.el"
        "byte-compiler support for inlining"
        (internal  dumped)
        "lisp/")
    ("bytecomp.el"
        "compilation of Lisp code into byte code."
        (internal)
        "lisp/")
    ("callers-of-rpt.el"
        "generate call graph of lisp in XEmacs"
        (internal)
        "lisp/")
    ("check-features.el"
        "Do a sanity check on an XEmacs build"
        (internal)
        "lisp/")
    ("cl-compat.el"
        "Common Lisp extensions for GNU Emacs Lisp (compatibility)"
        (extensions)
        "lisp/")
    ("cl-extra.el"
        "Common Lisp extensions for GNU Emacs Lisp (part two)"
        (extensions  dumped)
        "lisp/")
    ("cl-macs.el"
        "Common Lisp extensions for GNU Emacs Lisp (part four)"
        (extensions)
        "lisp/")
    ("cl-seq.el"
        "Common Lisp extensions for GNU Emacs Lisp (part three)"
        (extensions  dumped)
        "lisp/")
    ("cl.el"
        "Common Lisp extensions for GNU Emacs Lisp"
        (extensions  dumped  lisp)
        "lisp/")
    ("cleantree.el"
        "Remove out of date .elcs in lisp directories"
        (internal)
        "lisp/")
    ("cmdloop.el"
        "support functions for the top-level command loop."
        (internal  dumped)
        "lisp/")
    ("code-files.el"
        "File I/O functions for XEmacs."
        nil
        "lisp/")
    ("code-process.el"
        "Process coding functions for XEmacs."
        (mule  multilingual  coding system  process)
        "lisp/")
    ("coding.el"
        "Coding-system functions for XEmacs."
        nil
        "lisp/")
    ("config.el"
        "access configuration parameters"
        (configure)
        "lisp/")
    ("console.el"
        "miscellaneous console functions not written in C"
        (internal  dumped)
        "lisp/")
    ("cus-dep.el"
        "Find customization dependencies."
        (internal)
        "lisp/")
    ("cus-edit.el"
        "Tools for customizating Emacs and Lisp packages."
        (help  faces)
        "lisp/")
    ("cus-face.el"
        "Support for Custom faces."
        (help  faces)
        "lisp/")
    ("cus-load.el"
        "Batch load all available cus-load files"
        (internal  help  faces)
        "lisp/")
    ("cus-start.el"
        "define customization properties of builtins."
        (internal  dumped)
        "lisp/")
    ("custom-load.el"
        "automatically extracted custom dependencies"
        nil
        "lisp/")
    ("custom.el"
        "Tools for declaring and initializing options."
        (help  faces  dumped)
        "lisp/")
    ("derived.el"
        "allow inheritance of major modes."
        (extensions  dumped)
        "lisp/")
    ("device.el"
        "miscellaneous device functions not written in C"
        (internal  dumped)
        "lisp/")
    ("dialog.el"
        "Dialog-box support for XEmacs"
        (extensions  internal  dumped)
        "lisp/")
    ("disass.el"
        "disassembler for compiled Emacs Lisp code"
        (internal)
        "lisp/")
    ("disp-table.el"
        "functions for dealing with char tables."
        (i18n  internal)
        "lisp/")
    ("dragdrop.el"
        "window system-independent Drag'n'Drop support."
        (drag  drop  dumped)
        "lisp/")
    ("dump-paths.el"
        "set up XEmacs paths for dumping"
        (internal  dumped)
        "lisp/")
    ("etags.el"
        "etags facility for Emacs"
        (tools)
        "lisp/")
    ("events.el"
        "event functions for XEmacs."
        (internal  event  dumped)
        "lisp/")
    ("extents.el"
        "miscellaneous extent functions not written in C"
        (internal  dumped)
        "lisp/")
    ("faces.el"
        "Lisp interface to the C \"face\" structure"
        (faces  internal  dumped)
        "lisp/")
    ("files-nomule.el"
        "file I/O stubs when not under Mule."
        (extensions  dumped)
        "lisp/")
    ("files.el"
        "file input and output commands for XEmacs."
        (extensions  dumped)
        "lisp/")
    ("fill.el"
        "fill commands for XEmacs."
        (wp  dumped)
        "lisp/")
    ("find-paths.el"
        "setup various XEmacs paths"
        (internal  dumped)
        "lisp/")
    ("finder.el"
        "topic & keyword-based code finder"
        (help)
        "lisp/")
    ("float-sup.el"
        "detect absence of floating-point support in XEmacs runtime"
        (internal  dumped)
        "lisp/")
    ("font-lock.el"
        "decorating source files with fonts/colors based on syntax"
        (languages  faces)
        "lisp/")
    ("font.el"
        "New font model"
        (faces)
        "lisp/")
    ("fontl-hooks.el"
        "pre-loaded stuff for font-lock."
        nil
        "lisp/")
    ("format.el"
        "read and save files in multiple formats"
        (extensions  dumped)
        "lisp/")
    ("frame.el"
        "multi-frame management independent of window systems."
        (internal  dumped)
        "lisp/")
    ("glyphs.el"
        "Lisp interface to C glyphs"
        (extensions  internal  dumped)
        "lisp/")
    ("gnuserv.el"
        "Lisp interface code between Emacs and gnuserv"
        (environment  processes  terminals)
        "lisp/")
    ("gui.el"
        "Basic GUI functions for XEmacs."
        (internal  dumped)
        "lisp/")
    ("help-macro.el"
        "Makes command line help such as help-for-help"
        nil
        "lisp/")
    ("help-nomule.el"
        "Help functions when not in Mule"
        (help  internal  dumped)
        "lisp/")
    ("help.el"
        "help commands for XEmacs."
        (help  internal  dumped)
        "lisp/")
    ("hyper-apropos.el"
        "Hypertext emacs lisp documentation interface."
        (lisp  tools  help  docs  matching)
        "lisp/")
    ("indent.el"
        "indentation commands for XEmacs"
        (lisp  languages  tools  dumped)
        "lisp/")
    ("info.el"
        "info package for Emacs."
        (help)
        "lisp/")
    ("isearch-mode.el"
        "Incremental search minor mode."
        (extensions  dumped)
        "lisp/")
    ("iso8859-1.el"
        "Set case and syntax tables for Latin 1"
        (internal  dumped)
        "lisp/")
    ("itimer-autosave.el"
        "Autosave functions with itimers"
        (internal  dumped)
        "lisp/")
    ("keydefs.el"
        "Define standard keybindings."
        (internal  dumped)
        "lisp/")
    ("keymap.el"
        "Keymap functions for XEmacs."
        (internals  dumped)
        "lisp/")
    ("ldap.el"
        "LDAP support for Emacs"
        (help comm)
        "lisp/")
    ("lib-complete.el"
        "Completion on the lisp search path"
        (lisp  extensions  dumped)
        "lisp/")
    ("lisp-mnt.el"
        "minor mode for Emacs Lisp maintainers"
        (docs  maint)
        "lisp/")
    ("lisp-mode.el"
        "Lisp mode, and its idiosyncratic commands."
        (lisp  languages  dumped)
        "lisp/")
    ("lisp.el"
        "Lisp editing commands for XEmacs"
        (lisp  languages  dumped)
        "lisp/")
    ("list-mode.el"
        "Major mode for buffers containing lists of items"
        (extensions  dumped)
        "lisp/")
    ("loaddefs.el"
        "define standard autoloads of other files"
        (internal)
        "lisp/")
    ("loadhist.el"
        "lisp functions for working with feature groups"
        (internal  dumped)
        "lisp/")
    ("loadup.el"
        "load up standardly loaded Lisp files for XEmacs."
        (internal  dumped)
        "lisp/")
    ("make-docfile.el"
        "Cache docstrings in external file"
        (internal)
        "lisp/")
    ("map-ynp.el"
        "General-purpose boolean question-asker."
        (lisp  extensions  dumped)
        "lisp/")
    ("menubar-items.el"
        "Menubar and popup-menu content for XEmacs."
        (frames  extensions  internal  dumped)
        "lisp/")
    ("menubar.el"
        "Menubar support for XEmacs"
        (internal  extensions  dumped)
        "lisp/")
    ("minibuf.el"
        "Minibuffer functions for XEmacs"
        (internal  dumped)
        "lisp/")
    ("misc.el"
        "miscellaneous functions for XEmacs"
        (extensions  dumped)
        "lisp/")
    ("mode-motion.el"
        "Mode-specific mouse-highlighting of text."
        (internal  mouse  dumped)
        "lisp/")
    ("modeline.el"
        "modeline hackery."
        (extensions  dumped)
        "lisp/")
    ("mouse.el"
        "window system-independent mouse support."
        (mouse  dumped)
        "lisp/")
    ("msw-faces.el"
        "mswindows-specific face stuff."
        nil
        "lisp/")
    ("msw-glyphs.el"
        "Support for glyphs in ms windows"
        (extensions  internal  dumped)
        "lisp/")
    ("msw-init.el"
        "initialization code for mswindows"
        nil
        "lisp/")
    ("msw-mouse.el"
        "Mouse support for MS Windows."
        (mouse  dumped)
        "lisp/")
    ("msw-select.el"
        "Lisp interface to mswindows selections."
        (extensions  dumped)
        "lisp/")
    ("multicast.el"
        "lisp frontend for multicast connections in XEmacs"
        (dumped comm processes)
        "lisp/")
    ("mwheel.el"
        "Mouse support for MS intelli-mouse type mice"
        (mouse)
        "lisp/")
    ("objects.el"
        "Lisp interface to C window-system objects"
        (faces  internal  dumped)
        "lisp/")
    ("obsolete.el"
        "obsoleteness support"
        (internal  dumped)
        "lisp/")
    ("package-admin.el"
        "Installation and Maintenance of XEmacs packages"
        (internal)
        "lisp/")
    ("package-get.el"
        "Retrieve XEmacs package"
        (internal)
        "lisp/")
    ("package-info.el"
        "Generate information about an XEmacs package"
        (internal)
        "lisp/")
    ("packages.el"
        "Low level support for XEmacs packages"
        (internal  lisp  dumped)
        "lisp/")
    ("page.el"
        "page motion commands for emacs."
        (extensions  dumped)
        "lisp/")
    ("paragraphs.el"
        "paragraph and sentence parsing."
        (wp  dumped)
        "lisp/")
    ("paths.el"
        "define pathnames for use by various Emacs commands."
        (internal  dumped)
        "lisp/")
    ("picture.el"
        "\"Picture mode\" -- editing using quarter-plane screen model."
        nil
        "lisp/")
    ("process.el"
        "commands for subprocesses; split out of simple.el"
        (internal  processes  dumped)
        "lisp/")
    ("rect.el"
        "rectangle functions for XEmacs."
        (internal)
        "lisp/")
    ("register.el"
        "register commands for Emacs."
        (internal  dumped)
        "lisp/")
    ("replace.el"
        "search and replace commands for XEmacs."
        (dumped  matching)
        "lisp/")
    ("scrollbar.el"
        "Scrollbar support for XEmacs"
        (internal  extensions  dumped)
        "lisp/")
    ("select.el"
        "Lisp interface to windows selections."
        (extensions  dumped)
        "lisp/")
    ("setup-paths.el"
        "setup various XEmacs paths"
        (internal  dumped)
        "lisp/")
    ("shadow.el"
        "Locate Emacs Lisp file shadowings."
        (lisp)
        "lisp/")
    ("simple.el"
        "basic editing commands for XEmacs"
        (lisp  extensions  internal  dumped)
        "lisp/")
    ("site-load.el"
        "Template file for site-wide XEmacs customization"
        (internal)
        "lisp/")
    ("sound.el"
        "Loading sound files in XEmacs"
        (internal)
        "lisp/")
    ("specifier.el"
        "Lisp interface to specifiers"
        (internal  dumped)
        "lisp/")
    ("startup.el"
        "process XEmacs shell arguments"
        (internal  dumped)
        "lisp/")
    ("subr.el"
        "basic lisp subroutines for XEmacs"
        (extensions  dumped)
        "lisp/")
    ("symbol-syntax.el"
        "find chars with symbol syntax"
        (matching)
        "lisp/")
    ("symbols.el"
        "functions for working with symbols and symbol values"
        (internal)
        "lisp/")
    ("syntax.el"
        "Syntax-table hacking stuff, moved from syntax.c"
        nil
        "lisp/")
    ("text-mode.el"
        "text mode, and its idiosyncratic commands."
        (wp  dumped)
        "lisp/")
    ("text-props.el"
        "implements properties of characters"
        (extensions  wp  faces  dumped)
        "lisp/")
    ("toolbar-items.el"
        "Static initialization of XEmacs toolbar"
        (frames  dumped)
        "lisp/")
    ("toolbar.el"
        "Toolbar support for XEmacs"
        (extensions  internal  dumped)
        "lisp/")
    ("tty-init.el"
        "initialization code for tty's"
        (terminals  dumped)
        "lisp/")
    ("undo-stack.el"
        "An \"undoable stack\" object."
        (extensions  dumped)
        "lisp/")
    ("update-elc.el"
        "Bytecompile out-of-date dumped files"
        (internal)
        "lisp/")
    ("userlock.el"
        "handle file access contention between multiple users"
        (internal)
        "lisp/")
    ("version.el"
        "Record version number of Emacs."
        (internal  dumped)
        "lisp/")
    ("very-early-lisp.el"
        "Lisp support always needed by temacs"
        (internal  dumped)
        "lisp/")
    ("view-less.el"
        "Minor mode for browsing files with keybindings like `less'"
        (wp  unix)
        "lisp/")
    ("wid-browse.el"
        "Functions for browsing widgets."
        (extensions)
        "lisp/")
    ("wid-edit.el"
        "Functions for creating and using widgets."
        (extensions)
        "lisp/")
    ("widget.el"
        "a library of user interface components."
        (help  extensions  faces  hypermedia  dumped)
        "lisp/")
    ("window-xemacs.el"
        "XEmacs window commands aside from those written in C."
        (frames  extensions  dumped)
        "lisp/")
    ("window.el"
        "XEmacs window commands aside from those written in C."
        (frames  extensions  dumped)
        "lisp/")
    ("winnt.el"
        "Lisp routines for Windows NT."
        (mouse  dumped)
        "lisp/")
    ("x-compose.el"
        "Compose-key processing in XEmacs"
        (i18n)
        "lisp/")
    ("x-faces.el"
        "X-specific face frobnication, aka black magic."
        (extensions  internal  dumped)
        "lisp/")
    ("x-font-menu.el"
        "Managing menus of X fonts."
        nil
        "lisp/")
    ("x-init.el"
        "initialization code for X windows"
        (terminals  dumped)
        "lisp/")
    ("x-iso8859-1.el"
        "Mapping between X keysym names and ISO 8859-1"
        (extensions  internal  dumped)
        "lisp/")
    ("x-misc.el"
        "miscellaneous X functions."
        (extensions  dumped)
        "lisp/")
    ("x-mouse.el"
        "Mouse support for X window system."
        (mouse  dumped)
        "lisp/")
    ("x-scrollbar.el"
        "scrollbar resourcing and such."
        (extensions  dumped)
        "lisp/")
    ("x-select.el"
        "Lisp interface to X Selections."
        (extensions  dumped)
        "lisp/")
    ("x-win-sun.el"
        "runtime initialization for Sun X servers and keyboards"
        (terminals)
        "lisp/")
    ("x-win-xfree86.el"
        "runtime initialization for XFree86 servers"
        (terminals)
        "lisp/")
    ("arabic.el"
        "pre-loaded support for Arabic."
        nil
        "lisp/mule/")
    ("canna-leim.el"
        "Canna-related code for LEIM"
        (japanese  input method  leim)
        "lisp/mule/")
    ("chinese.el"
        "Support for Chinese"
        (multilingual  chinese)
        "lisp/mule/")
    ("cyrillic.el"
        "Support for languages which use Cyrillic characters"
        (multilingual  cyrillic)
        "lisp/mule/")
    ("english.el"
        "English support"
        (multibyte character  character set  syntax  category)
        "lisp/mule/")
    ("european.el"
        "Support for European languages"
        (multilingual  european)
        "lisp/mule/")
    ("greek.el"
        "Support for Greek"
        (multilingual  greek)
        "lisp/mule/")
    ("hebrew.el"
        "Support for Hebrew"
        (multilingual  hebrew)
        "lisp/mule/")
    ("japanese.el"
        "Japanese support"
        (multilingual  japanese)
        "lisp/mule/")
    ("kinsoku.el"
        "Kinsoku (line wrap) processing for XEmacs/Mule"
        nil
        "lisp/mule/")
    ("korean.el"
        "Support for Korean"
        (multilingual  korean)
        "lisp/mule/")
    ("misc-lang.el"
        "support for miscellaneous languages (characters)"
        (multilingual  character set  coding system)
        "lisp/mule/")
    ("mule-category.el"
        "category functions for XEmacs/Mule."
        nil
        "lisp/mule/")
    ("mule-ccl.el"
        "CCL (Code Conversion Language) compiler"
        (ccl  mule  multilingual  character set  coding-system)
        "lisp/mule/")
    ("mule-charset.el"
        "Charset functions for Mule."
        nil
        "lisp/mule/")
    ("mule-cmds.el"
        "Commands for multilingual environment"
        (mule  multilingual)
        "lisp/mule/")
    ("mule-coding.el"
        "Coding-system functions for Mule."
        nil
        "lisp/mule/")
    ("mule-files.el"
        "File I/O functions for XEmacs/Mule."
        nil
        "lisp/mule/")
    ("mule-help.el"
        "Mule-ized Help functions "
        (help  internal)
        "lisp/mule/")
    ("mule-misc.el"
        "Miscellaneous Mule functions."
        nil
        "lisp/mule/")
    ("mule-tty-init.el"
        "Initialization code for console tty under MULE"
        (mule  tty  console  dumped)
        "lisp/mule/")
    ("mule-x-init.el"
        "initialization code for X Windows under MULE"
        (mule x11)
        "lisp/mule/")
    ("viet-chars.el"
        "pre-loaded support for Vietnamese, part 1."
        nil
        "lisp/mule/")
    ("AT386.el"
        "terminal support package for IBM AT keyboards"
        (terminals)
        "lisp/term/")
    ("apollo.el"
        "Apollo Graphics Primitive Support Functions"
        (hardware)
        "lisp/term/")
    ("bg-mouse.el"
        "GNU Emacs code for BBN Bitgraph mouse."
        (hardware)
        "lisp/term/")
    ("internal.el"
        "setup support for PC keyboards and screens, internal terminal"
        nil
        "lisp/term/")
    ("keyswap.el"
        "swap BS and DEL keys"
        (terminals)
        "lisp/term/")
    ("linux.el"
        "define function key sequences for the Linux console"
        (terminals)
        "lisp/term/")
    ("lk201.el"
        "basic editing commands for XEmacs"
        nil
        "lisp/term/")
    ("news.el"
        "keypad and function key bindings for the Sony NEWS keyboard"
        (terminals)
        "lisp/term/")
    ("pc-win.el"
        "setup support for `PC windows' (whatever that is)."
        nil
        "lisp/term/")
    ("scoansi.el"
        "set up key names for SCO ansi console"
        nil
        "lisp/term/")
    ("sun-mouse.el"
        "mouse handling for Sun windows"
        (hardware)
        "lisp/term/")
    ("sun.el"
        "keybinding for standard default sunterm keys"
        (terminals)
        "lisp/term/")
    ("sup-mouse.el"
        "supdup mouse support for lisp machines"
        (hardware)
        "lisp/term/")
    ("tvi970.el"
        "terminal support for the Televideo 970"
        (terminals)
        "lisp/term/")
    ("vt-control.el"
        "Common VTxxx control functions"
        (terminals)
        "lisp/term/")
    ("vt100-led.el"
        "functions for LED control on VT-100 terminals & clones."
        (hardware)
        "lisp/term/")
    ("vt100.el"
        "define VT100 function key sequences in function-key-map"
        (terminals)
        "lisp/term/")
    ("win32-win.el"
        "parse switches controlling interface with win32"
        (terminals)
        "lisp/term/")
    ("wyse50.el"
        "terminal support code for Wyse 50"
        (terminals)
        "lisp/term/")
    ("xterm.el"
        "define function key sequences for xterm"
        (terminals)
        "lisp/term/")
))

(provide 'finder-inf)

;;; finder-inf.el ends here
